#include <bits/stdc++.h>

using namespace std;

struct Point {
    long long x, y;
    int id;

    Point(long long x, long long y, int id) : x(x), y(y), id(id) {}

    Point() {}

    Point(Point &a, Point &b) : x(b.x - a.x), y(b.y - a.y), id(-1) {}
};

long long cross(Point a, Point b) {
    return a.x * b.y - a.y * b.x;
}

long long dist(Point &a, Point &b) {
    return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y);
}

long long dot(Point &a, Point &b) {
    return a.x * b.x + a.y * b.y;
}

int n;
vector<Point> coords;
vector<Point> chull;
Point st;
int qs, qs1;
long long tot = 0;

#ifdef DEBUG
Point tiger;
#endif

bool cmp(Point &a, Point &b) {
    long long c = cross(Point(st, a), Point(st, b));
    if (c != 0) return c > 0;
    return dist(st, a) < dist(st, b);
}

void convex_hull() {
    st = coords[0];
    for (int i = 0; i < coords.size(); i++) {
        if (coords[i].y < st.y || (coords[i].y == st.y && coords[i].x < st.x)) {
            st = coords[i];
        }
    }
    sort(coords.begin(), coords.end(), cmp);

    for (int i = 0; i < coords.size(); i++) {
        while (chull.size() >= 2 && cross(Point(chull[chull.size() - 2], chull[chull.size() - 1]),
                                        Point(chull[chull.size() - 1], coords[i])) <= 0) {
            chull.pop_back();
        }
        chull.push_back(coords[i]);
    }
}

int sign(long long x) {
    if (x < 0) return -1;
    if (x > 0) return 1;
    return 0;
}

bool is_in(Point &p, Point &a, Point &b, Point &c) {
    Point v1(a, b), v2(b, c), v3(c, a);
    Point u1(a, p), u2(b, p), u3(c, p);
    int s1 = sign(cross(v1, u1));
    int s2 = sign(cross(v2, u2));
    int s3 = sign(cross(v3, u3));
    return s1 == s2 && s2 == s3;
}

long double angle(Point a, Point b) {
    return atan2(cross(a, b), dot(a, b));
}

bool ask(vector<Point> &p) {
    qs += p.size();
    qs1++;

    if (qs > 100000) {
        exit(123);
    }
    if (qs1 > 40) {
        exit(234);
    }
#ifndef DEBUG
    cout << "? " << p.size();
    for (auto pp : p) {
        cout << " " << pp.id;
    }
    cout << endl;
    string ans;
    cin >> ans;
    return ans == "Yes";
#else
    long double sm = 0;
    for (int i = 0; i < p.size(); i++) {
        sm += angle(Point(tiger, p[i]), Point(tiger, p[(i + 1) % p.size()]));
    }
    return sm > 0.1;
#endif
}

void find() {
    // In polygon {0, L, L + 1, L + 2, ..., R - 1, R} there is a tiger
    int L = 1;
    int R = chull.size() - 1;
    while (R - L > 1) {
        int M = (L + R) / 2;

        vector<Point> cur;
        cur.push_back(chull[0]);
        for (int i = L; i <= M; i++) {
            cur.push_back(chull[i]);
        }

        if (ask(cur)) {
            R = M;
        } else {
            L = M;
        }
    }

    // Tiger is in triangle {0, L, L + 1}
    Point a = chull[0];
    Point b = chull[L];
    Point c = chull[L + 1];
//    cout << "OK " << a.id << " " << b.id << " " << c.id << endl;

    vector<Point> pts;
    for (Point p : coords) {
        if (p.id != a.id && p.id != b.id && p.id != c.id && is_in(p, a, b, c)) {
            pts.push_back(p);
        }
    }

    while (pts.size() > 0) {

        int best = 1000000000;
        Point bp = pts[0];

        for (auto pp : pts) {
            int cntab = 0, cntac = 0, cntbc = 0;
            for (auto p : pts) {
                tot++;
                if (p.id == pp.id) continue;
                if (is_in(p, a, b, pp)) {
                    cntab++;
                } else if (is_in(p, b, c, pp)) {
                    cntbc++;
                } else if (is_in(p, c, a, pp)) {
                    cntac++;
                } else {
                    exit(239);
                }
            }

            int mx = max(max(cntab, cntbc), cntac);

            if (mx < best) {
                best = mx;
                bp = pp;
            }

            if (best <= pts.size() / 2 + 10) {
                break;
            }
        }

        string ans;

        vector<Point> qq(3);
        qq[0] = a;
        qq[1] = b;
        qq[2] = bp;
        if (ask(qq)) {
            c = bp;
        } else {
            qq[0] = b;
            qq[1] = c;
            qq[2] = bp;
            if (ask(qq)) {
                a = bp;
            } else {
                b = bp;
            }
        }

        vector<Point> newpts;
        for (auto p : pts) {
            if (p.id != a.id && p.id != b.id && p.id != c.id && is_in(p, a, b, c)) {
                newpts.push_back(p);
            }
        }

        pts.swap(newpts);
    }

    cout << "! 3 " << a.id << " " << b.id << " " << c.id << endl;
}

int main() {
#ifdef DEBUG
    freopen("input", "r", stdin);
    freopen("output", "w", stdout);
    long double start = clock();
#endif

    ios::sync_with_stdio(0);
    cin.tie(0);

    cin >> n;
    coords.resize(n);
    for (int i = 0; i < n; i++) {
        cin >> coords[i].x >> coords[i].y;
        coords[i].id = i + 1;
    }
    convex_hull();

    int q;
    cin >> q;
    for (int i = 0; i < q; i++) {
        qs = qs1 = 0;
#ifdef DEBUG
        if (i % 50 == 49) {
            cerr << i << " " << (clock() - start) / CLOCKS_PER_SEC << " " << tot << endl;
        }
        cin >> tiger.x >> tiger.y;
#endif
        find();
    }

    return 0;
}

/*

6
3 0
0 2
2 4
4 2
5 5
6 1
4

 */