#include <iostream>
#include <vector>
#include <queue>
#include <cstdlib>
#include <string>
#include <stack>
#include <algorithm>
#include <cmath>
#include <unordered_set>

using namespace std;

struct vec
{
  double x, y;

  vec(double x = 0, double y = 0) : x(x), y(y)
  {
  }

  vec operator+(const vec & v)
  {
    return vec(x + v.x, y + v.y);
  }
  vec & operator+=(const vec & v)
  {
    x += v.x;
    y += v.y;
    return (*this);
  }
  vec operator-(const vec & v)
  {
    return vec(x - v.x, y - v.y);
  }
  vec & operator-=(const vec & v)
  {
    x -= v.x;
    y -= v.y;
    return (*this);
  }
  double operator%(const vec & v)
  {
    return x * v.y - v.x * y;
  }
  double operator*(const vec & v)
  {
    return x * v.x + y * v.y;
  }
  vec operator*(double a)
  {
    return vec(x * a, y * a);
  }
  vec & operator*=(double a)
  {
    x *= a;
    y *= a;
    return (*this);
  }
  bool is_left(const vec & v)
  {
    return ((*this) % v > 1e-9);
  }
  bool is_right(const vec & v)
  {
    return ((*this) % v < -1e-9);
  }
  double getan(void)
  {
    return atan2(y, x);
  }
};

int main(void)
{
  int n;
  ios::sync_with_stdio(0);
  srand(30);
  cin >> n;
  vector<vec> pts(n);
  vector<int> ind(n);
  for (int i = 0; i < n; i++)
  {
    double xj, yj;
    cin >> xj >> yj;
    pts[i] = vec(xj, yj);
    ind[i] = i;
  }
  sort(ind.begin(), ind.end(), [&pts](int a, int b)->bool { return pts[a].x > pts[b].x + 1e-9 || (fabs(pts[a].x - pts[b].x) < 1e-9 && pts[a].y < pts[b].y - 1e-9); });
  int mlt = ind.back();
  ind.pop_back();
  sort(ind.begin(), ind.end(), [&pts, &mlt](int a, int b)->bool { return (pts[b] - pts[mlt]).getan() > (pts[a] - pts[mlt]).getan() + 1e-9;  });
  vector<int> conv_hull(1, mlt);
  for (auto i : ind)
  {
    if (conv_hull.size() == 1)
    {
      conv_hull.push_back(i);
      continue;
    }
    while ((pts[conv_hull[conv_hull.size() - 1]] - pts[conv_hull[conv_hull.size() - 2]]).is_right(pts[i] - pts[conv_hull[conv_hull.size() - 1]]))
      conv_hull.pop_back();
    conv_hull.push_back(i);
  }
  int q;
  cin >> q;
  for (int i = 0; i < q; i++)
  {
    vector<int> ncv = conv_hull;
    while (ncv.size() > 3)
    {
      int m = ncv.size() / 2;
      cout << "? " << m + 1 << " ";
      for (int j = 0; j <= m; j++)
        cout << ncv[j] + 1 << " ";
      cout << endl;
      string ans;
      cin >> ans;
      if (ans == "Yes")
        ncv.resize(m + 1);
      else
      {
        ncv.push_back(ncv[0]);
        ncv.erase(ncv.begin(), ncv.begin() + m);
      }
    }
    unordered_set<int> curcand;
    for (int j = 0; j < n; j++)
      curcand.insert(j);
    while (curcand.size() != 0)
    {
      int j = *(curcand.begin());
      bool flag = true;
      for (int k = 0; k < 3; k++)
        if (!((pts[ncv[(k + 1) % 3]] - pts[ncv[k]]).is_left(pts[j] - pts[ncv[k]])))
        {
          flag = false;
          break;
        }
      if (!flag)
      {
        curcand.erase(curcand.begin());
        continue;
      }
      cout << "? 3 " << ncv[0] + 1 << " " << ncv[1] + 1 << " " << j + 1 << endl;
      string ans;
      cin >> ans;
      if (ans == "Yes")
      {
        ncv[2] = j;
        continue;
      }
      cout << "? 3 " << ncv[1] + 1 << " " << ncv[2] + 1 << " " << j + 1 << endl;
      cin >> ans;
      if (ans == "Yes")
      {
        ncv[0] = j;
        continue;
      }
      ncv[1] = j;
    }
    cout << "! " << ncv[0] + 1 << " " << ncv[1] + 1 << " " << ncv[2] + 1 << endl;
  }
  return 0;
}