#include<iostream>
#include<vector>
#include<string>
#include<algorithm>
#include<cassert>

#define int long long

using namespace std;

struct point {
	int x, y, id;
	point() {}
	point(int x, int y, int id) : x(x), y(y), id(id) {}
};

struct vect {
	int x, y;
	vect(int x, int y) : x(x), y(y) {}
	vect(point a, point b) : x(b.x - a.x), y(b.y - a.y) {}
	int operator^(const vect& b) const {
		return x * b.y - y * b.x;
	}
	int len2() {
		return x * x + y * y;
	}
};

vector<point> ps;

int ask(vector<int> &req) {
	int sz = req.size();
	cout << "? " << sz << " ";
	for (int i = 0; i < sz; i++) {
		cout << req[i] + 1;
		if (i == sz - 1)
			cout << endl;
		else
			cout << " ";
	}
	string res;
	cin >> res;
	return res == "Yes";
}

void answer(int a1, int a2, int a3) {
	cout << "! 3 " << a1 + 1 << " " << a2 + 1 << " " << a3 + 1 << endl;
	//exit(0);
}

int sgn(int x) {
	return (x > 0) - (x < 0);
}

bool inside(point x, point a, point b, point c) {
	vect ab(a, b), ax(a, x);
	vect bc(b, c), bx(b, x);
	vect ca(c, a), cx(c, x);
	int sa = sgn(ab ^ ax), sb = sgn(bc ^ bx), sc = sgn(ca ^ cx);
	if (sa == 0 || sb == 0 || sc == 0)
		return false;
	return (sa == sb && sb == sc);
}

vector<point> hull;
int k;
int n;
int smart_ask(int m) {
	vector<int> req;
	for (int i = 0; i <= m; i++) {
		req.push_back(hull[i].id);
	}
	return ask(req);
}

int task(int a1, int a2, int a3) {
	cout << "? 3 " << a1 + 1 << " " << a2 + 1 << " " << a3 + 1 << endl;
	string res;
	cin >> res;
	return res == "Yes";
}

void work(point a1, point a2, point a3, vector<point> &cps) {
	vector<point> ins;
	for (int i = 0; i < cps.size(); i++) {
		if (inside(cps[i], a1, a2, a3))
			ins.push_back(cps[i]);
	}
	if (ins.size() == 0) {
		answer(a1.id, a2.id, a3.id);
		return;
	}
	point r = ins[rand() % ins.size()];

	if (task(a1.id, a2.id, r.id))
		work(a1, a2, r, ins);
	else if (task(a2.id, a3.id, r.id))
		work(a2, a3, r, ins);
	else
		work(a3, a1, r, ins);
}

point zero;
bool polar_cmp(point a, point b) {
	vect za(zero, a), zb(zero, b);
	int s = za ^ zb;
	if (s != 0)
		return (za ^ zb) > 0;
	return za.len2() < zb.len2();
}

void get_hull(vector<point> &ps) {
	zero = ps[0];
	for (int i = 1; i < n; i++) {
		if (ps[i].y < zero.y || (ps[i].y == zero.y && ps[i].x < zero.x))
			zero = ps[i];
	}
	sort(ps.begin(), ps.end(), polar_cmp);
	hull.push_back(ps[0]);
	hull.push_back(ps[1]);
	for (int i = 2; i < n; i++) {
		while (hull.size() > 1 && (vect(hull[hull.size() - 2], hull[hull.size() - 1])
			^ vect(hull[hull.size() - 1], ps[i])) < 0)
			hull.pop_back();
		hull.push_back(ps[i]);
	}
}

int32_t main() {
	//freopen("input.in", "r", stdin);
	srand(2539);
	ios_base::sync_with_stdio(false);
	cin >> n;
	cin.tie(0);
	ps.resize(n);
	for (int i = 0; i < n; i++) {
		cin >> ps[i].x >> ps[i].y;
		ps[i].id = i;
	}
	int q;
	cin >> q;
	get_hull(ps);
	k = hull.size();
	for (int cq = 0; cq < q; cq++) {
		int l = 1, r = k - 1;
		while (r - l > 1) {
			int m = l + r >> 1;
			int a1 = smart_ask(m);
			if (a1)
				r = m;
			else
				l = m;
		}
		work(hull[0], hull[r], hull[r - 1], ps);
	}
}