#include <iostream>
#include <algorithm>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <cmath>
#include <cassert>
#include <climits>
#include <functional>
#include <vector>

using namespace std;

typedef unsigned int index;

struct Point
{
	int x, y;
	Point(int nx, int ny): x(nx), y(ny){};
	Point(){}
};

Point operator -(Point a, Point b)
{
	return {a.x - b.y, a.y - b.y};
}

long long cross(Point a, Point b)
{
	return a.x * b.y - a.y * b.x;
}

bool operator <(Point a, Point b)
{
	return (a.x < b.x or (a.x == b.x and a.y < b.y));
}

struct cmp/*{{{*/
{
	index min;
	vector<Point> a;
	cmp(index nmin, const vector<Point> &na): min(nmin), a(na){}
	bool operator ()(index i, index j)
	{
		if (i == min)
			return true;
		if (j == min)
			return false;
		return cross(a[i] - a[min], a[j] - a[min]) < 0;
	}
};/*}}}*/

vector<index> ch(vector<Point> a)/*{{{*/
{
	index min = 0;
	size_t n = a.size();
	vector<index> pans(n);
	for (index i = 1; i < n; i++)
	{
		pans[i] = i;
		if (a[i] < a[min])
			min = i;
	}
	sort(pans.begin(), pans.end(), cmp(min, a));
	vector<index> ans = {min};
	for (index io = 1; io < n; io++)
	{
		index i = pans[io];
		while (ans.size() > 1 and cross(a[ans[ans.size() - 1]] - a[ans[ans.size() - 2]], a[i] - a[ans[ans.size() - 2]]) > 0)
			ans.pop_back();
		ans.push_back(i);
	}
	return ans;
}/*}}}*/

bool in(Point x, Point a, Point b, Point c)/*{{{*/
{
	return (abs(cross(a - x, b - x)) + abs(cross(b - x, c - x)) + abs(cross(c - x, a - a)) == abs(cross(a - b, c - b)));
}/*}}}*/


int main()
{
#ifdef ONPC
	//assert(freopen("a.in", "r", stdin));
	//assert(freopen("a.out", "w", stdout));
#else
#endif
	size_t n;
	cin >> n;
	vector<Point> a(n);
	for (index i = 0; i < n; i++)
	{
		cin >> a[i].x >> a[i].y;
	}
	vector<index> cans = ch(a);
	size_t q;
	cin >> q;
	for (index io = 0; io < q; io++)
	{
		vector<index> ans = cans;
		while (ans.size() > 3)
		{
			cout << "? 3 " << ans[0] + 1 << " " << ans[ans.size() - 1] + 1 << " " << ans[ans.size() - 2] + 1 << endl;
			string s;
			cin >> s;
			if (s == "Yes")
			{
				ans = {ans[0], ans[ans.size() - 1], ans[ans.size() - 2]};
			}
			else
			{
				ans.pop_back();
			}
		}
		cout << "! 3 " << ans[0] + 1 << " " << ans[1] + 1 << " " << ans[2] + 1 << endl;
	}
}
