#include <iostream>
#include <string>
#include <vector>
#include <bitset>
#include <random>
#include <algorithm>
using namespace std;

const int maxN = 5000 + 100;

mt19937_64 gen;

long long random(long long l, long long r)
{
	uniform_int_distribution<long long> dist(l, r);
	return dist(gen);
}

struct point
{
	long long x, y;

	point() {}
	point(long long x, long long y) : x(x), y(y) {}
};

struct vect
{
	long long x, y;
	vect() {}
	vect(long long x, long long y) : x(x), y(y) {}

	vect(point a, point b)
	{
		x = b.x - a.x;
		y = b.y - a.y;
	}

	long long operator% (vect b)
	{
		return x * b.y - y * b.x;
	}

	long long operator* (vect b)
	{
		return x * b.x + y * b.y;
	}
};

bool inside(point a, point b, point c, point d)
{
	long long area = abs(vect(a, b) % vect(a, c));
	long long tmp = abs(vect(a, b) % vect(a, d)) + abs(vect(a, d) % vect(a, c)) + abs(vect(b, d) % vect(b, c));
	return area == tmp;
}

point pts[maxN];

int lft = 0;

bool cmp(int i, int j)
{
	vect li(pts[lft], pts[i]), lj(pts[lft], pts[j]);
	return li % lj > 0;
}

vector<int> convex_hull(vector<int> cpts)
{
	lft = cpts.front();
	for (auto p : cpts)
	{
		if (pts[lft].x > pts[p].x)
		{
			lft = p;
		}
		else if (pts[lft].x == pts[p].x && pts[lft].y > pts[p].y)
		{
			lft = p;
		}
	}

	cpts.erase(find(cpts.begin(), cpts.end(), lft));

	sort(cpts.begin(), cpts.end(), cmp);
	vector<int> ch;
	ch.push_back(lft);
	for (auto p : cpts)
	{
		if (ch.size() < 2)
		{
			ch.push_back(p);
		}
		else
		{
			while (ch.size() >= 2)
			{
				int sz = ch.size();
				int p1 = ch[sz - 1], p2 = ch[sz - 2];
				if (vect(pts[p2], pts[p1]) % vect(pts[p1], pts[p]) > 0)
				{
					ch.push_back(p);
					break;
				}
				else
				{
					ch.pop_back();
				}
			}
		}
	}
	return ch;
}

bool check(vector<int> pts)
{
	cout << "? " << pts.size() << " ";
	for (int x : pts)
	{
		cout << x + 1 << " ";
	}
	cout << endl;
	string a;
	cin >> a;
	if (a == "Yes")
		return true;
	return false;
}

bool check(int a, int b, int c)
{
	cout << "? " << 3 << " " << a + 1 << " " << b + 1 << " " << c + 1;
	cout << endl;
	string ans;
	cin >> ans;
	if (ans == "Yes")
		return true;
	return false;
}



void solve(int a, int b, int c, vector<int> in)
{
	if (in.empty())
	{
		cout << "! 3 " << a + 1 << " " << b + 1 << " " << c + 1 << endl;
		return;
	}
	int pos = random(0, (int)in.size() - 1);
	int pivot = in[pos];
	if (check(a, b, pivot))
	{
		vector<int> tmp;
		for (int i = 0; i < in.size(); i++)
		{
			if (i == pos)
				continue;
			if (inside(pts[a], pts[b], pts[pivot], pts[in[i]]))
				tmp.push_back(in[i]);
		}
		solve(a, b, pivot, tmp);
	}
	else if (check(b, c, pivot))
	{
		vector<int> tmp;
		for (int i = 0; i < in.size(); i++)
		{
			if (i == pos)
				continue;
			if (inside(pts[b], pts[c], pts[pivot], pts[in[i]]))
				tmp.push_back(in[i]);
		}
		solve(b, c, pivot, tmp);
	}
	else
	{
		vector<int> tmp;
		for (int i = 0; i < in.size(); i++)
		{
			if (i == pos)
				continue;
			if (inside(pts[a], pts[c], pts[pivot], pts[in[i]]))
				tmp.push_back(in[i]);
		}
		solve(a, c, pivot, tmp);
	}
	return;
}

int main()
{
#ifdef _DEBUG
	//freopen("input.txt", "r", stdin);
#endif
	ios::sync_with_stdio(false);
	cin.tie(0);
	int n;
	cin >> n;

	vector<int> ch;
	for (int i = 0; i < n; i++)
	{
		long long x, y;
		cin >> x >> y;
		pts[i] = point(x, y);
		ch.push_back(i);
	}
	ch = convex_hull(ch);

	int q;
	cin >> q;

	for (int i = 0; i < q; i++)
	{
		int r = ch.size() - 1, l = 1;
		while (r - l > 1)
		{
			int m = (l + r) / 2;
			vector<int> tmp;
			for (int j = 0; j <= m; j++)
			{
				tmp.push_back(ch[j]);
			}
			if (check(tmp))
				r = m;
			else
				l = m;
		}
		vector<int> tmp;
		int a = ch[0], b = ch[r], c = ch[r - 1];
		for (int j = 0; j < n; j++)
		{
			if (j == a || j == b || j == c)
				continue;
			else if(inside(pts[a], pts[b], pts[c], pts[j]))
				tmp.push_back(j);
		}
		solve(a, b, c, tmp);
	}

    return 0;
}

