#include <iostream>
#include <vector>
#include <fstream>

using namespace std;

typedef long long ll;
typedef unsigned long long ull;

struct Vector {
    ll x, y;

    Vector(): x(0), y(0) {}
    Vector(ll x, ll y): x(x), y(y) {}

    Vector operator-(const Vector &other) const {
        return Vector(x - other.x, y - other.y);
    }

    Vector to(const Vector &other) const {
        return other - *this;
    }

    ll cross(const Vector &other) const {
        return x * other.y - y * other.x;
    }

    ll scal(const Vector &other) const {
        return x * other.x + y * other.y;
    }
};

istream& operator>>(istream &in, Vector &v) {
    in >> v.x >> v.y;
    return in;
}

ostream& operator<<(ostream &out, Vector v) {
    out << "(" << v.x << " " << v.y << ")";
    return out;
}

typedef Vector Vec;

const int MAX_N = 5000;

int n;
Vec pts[MAX_N];

ull triArea(int i1, int i2, int i3) {
    Vec p1 = pts[i1], p2 = pts[i2], p3 = pts[i3];
    return abs(p1.to(p2).cross(p1.to(p3)));
}

int sign_(ll x) {
    if (x > 0)
        return 1;
    return -1;
}

int side(Vec p, Vec p1, Vec p2) {
    return sign_(p.to(p1).cross(p.to(p2)));
}

bool inTri(Vec p, int i1, int i2, int i3) {
    Vec p1 = pts[i1], p2 = pts[i2], p3 = pts[i3];
    //cout << p << ": " << p1 << " " << p2 << " " << p3 << endl;
    int s1 = side(p, p1, p2);
    int s2 = side(p, p2, p3);
    int s3 = side(p, p3, p1);
    //cout << s1 << s2 << s3 << endl;
    return s1 == s2 && s2 == s3;
}

Vec tiger;

bool query(int i1, int i2, int i3) {
    cout << "? 3 " << i1 + 1 << " " << i2 + 1 << " " << i3 + 1 << endl;
    cout.flush();

    /*
    bool isIn = inTri(tiger, i1, i2, i3);
    cout << isIn << endl;
    return isIn;
    */
    string ans;
    cin >> ans;
    return ans[0] == 'Y';
}

void solve() {
    ull minArea = 0;
    int idx[] = {-1, -1, -1};
    for (int i1 = 0; i1 < n; i1++)
    for (int i2 = i1 + 1; i2 < n; i2++)
    for (int i3 = i2 + 1; i3 < n; i3++) {
        if (query(i1, i2, i3)) {
            ull na = triArea(i1, i2, i3);
            if (minArea == 0 || na < minArea) {
                minArea = na;
                idx[0] = i1;
                idx[1] = i2;
                idx[2] = i3;
            }
        }
    }

    cout << "! 3 ";
    for (int i = 0; i < 3; i++)
        cout << idx[i] + 1 << " ";
    cout << endl;
    cout.flush();
}

int main() {
    //ifstream fin("input.txt");

    cin >> n;
    for (int i = 0; i < n; i++) {
        cin >> pts[i];
    }
    int q;
    cin >> q;

    for (int i = 0; i < q; i++) {
        //fin >> tiger;
        //cout << inTri(tiger, 0, 1, 2);

        //break;
        solve();

    }
    //fin.close();
}
