// ConsoleApplication1.cpp : Defines the entry point for the console application.
//

#include <iostream>
#include <vector>
#include <string>
#include <cmath>
#include <algorithm>
#include <cstdio>
#include <ctime>
#include <cstdlib>

using namespace std;

struct Point {
	int x, y;
	int num;
	Point(): x(0), y(0), num(0) {}
	Point(int xy, int yx) : x(xy), y(yx), num(0) {}

	bool operator<(const Point &P) const {
		return y < P.y || (y == P.y && x < P.x);
	}

	double get_polar(Point p) {
		int xv = x - p.x;
		int vy = y - p.y;
		double d = atan2(vy, xv);

		if (d < 0)
			d += (double)2 * 3.1415;
		return d;
	}
};

int get_vector_mul(Point a, Point b) {
	return a.x * b.y - a.y * b.x;
}

auto xcmp = [](Point a, Point b) { return a.x < b.x;  };
auto ycmp = [](Point a, Point b) { return a.y < b.y;  };


vector<Point> build_hull(vector<Point> pts) {
	sort(pts.begin(), pts.end());

	auto acmp = [&pts](Point a, Point b) {
		return a.get_polar(pts.front()) < b.get_polar(pts.front());
	};

	stable_sort(pts.begin() + 1, pts.end(), acmp);
	vector<Point> hull;
	hull.push_back(pts[0]);
	hull.push_back(pts[1]);

	for (int i = 2; i < pts.size(); i++) {
		Point a = pts[i];
		Point b = hull[hull.size() - 1];
		Point c = hull[hull.size() - 2];
		while (hull.size() > 2 && get_vector_mul(Point(a.x - b.x, a.y - b.y), Point(b.x - c.x, b.y - c.y)) > 0)
			hull.pop_back();
		hull.push_back(pts[i]);
	}

	return hull;

}

bool ask(vector<Point> hull) {
	cout << "? ";
	for (auto it : hull) {
		cout << it.num + 1 << ' ';
	}
	cout << endl;
	string s;
	cin >> s;
	return s[0] == 'Y';

}

void ans(vector<Point> hull) {
	cout << "! ";
	for (auto it : hull) 
		cout << it.num + 1 << ' ';
	cout << endl;
}

void solve(vector<Point> trans, bool xmp) {

	if (trans.size() < 4) {
		ans(trans);
		return;
	}
	auto cmp = [](Point a, Point b) {return rand() & 1; };
	nth_element(trans.begin(), trans.begin(), trans.end(), cmp);
	nth_element(trans.begin() + 1, trans.end() - 1, trans.end(), cmp);

	Point P1 = trans.front();
	Point P2 = trans.back();


	vector<Point> top;
	vector<Point> bot;

	top.push_back(P1);
	top.push_back(P2);
	bot.push_back(P1);
	bot.push_back(P2);


	int a = P2.y - P1.y;
	int b = P1.x - P2.x;

	double c = double(P1.y * (P2.x - P1.x) - P1.x * (P2.y - P1.y)) / double(b);

	for (int i = 1; i < trans.size() - 1; i++) {
		Point p = trans[i];
		if (p.x * a + p.y * b < c)
			bot.push_back(p);
		else
			top.push_back(p);
	}

	vector<Point> tophull = build_hull(top);
	vector<Point> bothull = build_hull(bot);

	if (tophull.size() > 2 && ask(tophull)) {
		if (tophull.size() == top.size())
			ans(tophull);
		else
			solve(top, !xmp);
	}
	else {
		if (bothull.size() > 2 && bothull.size() == bot.size())
			ans(bothull);
		else
			solve(bot, !xmp);
	}

}

signed main()
{
	srand(time(nullptr));
	int n;
	cin >> n;

	vector<Point> trans(n);

	for (int i = 0; i < n; i++) {
		int x, y;
		cin >> x >> y;
		trans[i] = Point(x, y);
		trans[i].num = i;
	}

	int q;
	cin >> q;
	for (int i = 0; i < q; i++) {
		solve(trans, false);
	}
	



	return 0;
}

