#pragma comment(linker, "/Stack:512000000")
#define _CRT_SECURE_NO_WARNINGS

#include <iostream>
#include <cstdio>
#include <cmath>
#include <vector>
#include <string>
#include <cstring>
#include <map>
#include <set>
#include <algorithm>
#include <unordered_map>
#include <unordered_set>

using namespace std;

typedef long long LL;
typedef double dbl;

struct Line {
  LL k, b;
};

struct Point {
  dbl x, y;
  bool operator<(const Point &p) const {
    return fabs(x - p.x) < 1e-9 ? x < p.x : y < p.y;
  }
};

Point intersect(const Line &a, const Line &b) {
  dbl x = ((dbl)b.b - a.b) / ((dbl)a.k - b.k);
  return Point{ x, a.k * x + a.b };
}

bool isBelow(const Point &p, const Line &l) {
  return l.k * p.x > l.b;
}

struct CHT {
  vector<Point> points;
  vector<Line> lines;

  CHT() {
    lines.push_back({ (LL)-1e9, (LL)-1e9 });
  }

  void add(Line l) {
    while (!points.empty() && isBelow(points.back(), l)) points.pop_back(), lines.pop_back();
    points.push_back(intersect(l, lines.back()));
    lines.push_back(l);
  }

  Line get(LL x) {
    if (points.empty()) return Line{ 0, 0 };
    int j = lower_bound(points.begin(), points.end(), Point{ (dbl)x, -1e9 }) - points.begin() + 1;
    return lines[j];
  }
};

int main() {
#ifdef _DEBUG
  freopen("input.txt", "r", stdin);
  freopen("output.txt", "w", stdout);
#elif 1
# define PROB "trains"
  freopen(PROB".in", "r", stdin);
  freopen(PROB".out", "w", stdout);
#endif
  int n, m;
  scanf("%d%d", &n, &m);

  struct Edge {
    int from, to, cost, path, index;
  };

  vector<vector<Edge>> paths(m);
  vector<vector<LL>> pathsprefix(m);
  vector<vector<Edge>> g(n + 1);

  for (int i = 0; i < m; ++i) {
    int s;
    scanf("%d", &s);

    paths[i].reserve(s);
    pathsprefix[i].reserve(s + 1);
    pathsprefix[i].push_back(0);

    int prev;
    scanf("%d", &prev);

    LL sum = 0;

    for (int j = 0; j < s; ++j) {
      int cost, next;
      scanf("%d%d", &cost, &next);
      Edge e = Edge{ prev, next, cost, i, j };
      paths[i].push_back(e);
      sum += cost;
      pathsprefix[i].push_back(sum);
      g[prev].push_back(e);
      prev = next;
    }
  }

  vector<int> order;
  set<pair<int, int>> q;
  
  vector<int> dist(n + 1, (int)2e9);

  dist[1] = 0;
  q.insert({ 0, 1 });

  while (!q.empty()) {
    int v = q.begin()->second;
    order.push_back(v);
    q.erase(q.begin());

    for (auto to : g[v]) {
      if (dist[to.to] > dist[v] + to.cost) {
        q.erase({ dist[to.to], to.to });
        dist[to.to] = dist[v] + to.cost;
        q.insert({ dist[to.to], to.to });
      }
    }
  }

  vector<int> revorder = order;
  reverse(revorder.begin(), revorder.end());

  cout << dist[n] << ' ';

  vector<CHT> chts(m);
  vector<LL> dpmx(n + 1);
  vector<bool> good(n + 1);
  good[n] = 1;

  for (auto v : revorder) {
    for (auto to : g[v]) {
      if (!good[to.to]) continue;
      if (dist[v] + to.cost == dist[to.to]) {
        good[v] = 1;
        chts[to.path].add(Line{ -2 * pathsprefix[to.path][to.index + 1], dpmx[to.to] + pathsprefix[to.path][to.index + 1] * pathsprefix[to.path][to.index + 1] });

        LL &cur = dpmx[v];
        
        Line opt = chts[to.path].get(pathsprefix[to.path][to.index]);

        if (cur < opt.b + opt.k * pathsprefix[to.path][to.index] + pathsprefix[to.path][to.index] * pathsprefix[to.path][to.index]) {
          cur = opt.b + opt.k * pathsprefix[to.path][to.index] + pathsprefix[to.path][to.index] * pathsprefix[to.path][to.index];
        }

        if (cur < dpmx[to.to] + to.cost * to.cost) {
          cur = dpmx[to.to] + to.cost * to.cost;
        }
      }
      else {
        chts[to.path] = CHT();
      }
    }
  }

  cout << dpmx[1] << endl;
  
  return 0;
}